package org.jeecg.modules.adweb.serp.controller;

import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xkcoding.http.util.StringUtil;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;

import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

import lombok.extern.slf4j.Slf4j;

import org.apache.poi.ss.usermodel.Sheet;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.api.ISysBaseAPI;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.adweb.common.util.ListUtil;
import org.jeecg.modules.adweb.serp.dto.AvesApiSearchKeywordsDTO;
import org.jeecg.modules.adweb.serp.entity.SeoKeywords;
import org.jeecg.modules.adweb.serp.entity.SeoKpiStatistics;
import org.jeecg.modules.adweb.serp.service.ISearchKeywordsService;
import org.jeecg.modules.adweb.serp.service.ISeoKeywordsService;
import org.jeecg.modules.adweb.serp.service.ISeoKpiStatisticsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * @Description: 关键词
 * @Author: jeecg-boot
 * @Date:   2024-10-15
 * @Version: V1.0
 */
@Tag(name="关键词")
@RestController
@RequestMapping("/serp/seoKeywords")
@Slf4j
public class SeoKeywordsController extends JeecgController<SeoKeywords, ISeoKeywordsService> {
	@Autowired
	private ISeoKeywordsService seoKeywordsService;

	@Autowired
	ISeoKpiStatisticsService seoKpiStatisticsService;

	@Autowired
	ISearchKeywordsService searchKeywordsService;

	@Autowired
	private ISysBaseAPI sysBaseAPI;
	
	/**
	 * 分页列表查询
	 *
	 * @param seoKeywords
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "关键词-分页列表查询")
	@Operation(summary="关键词-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<SeoKeywords>> queryPageList(SeoKeywords seoKeywords,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<SeoKeywords> queryWrapper = QueryGenerator.initQueryWrapper(seoKeywords, req.getParameterMap());
		Page<SeoKeywords> page = new Page<SeoKeywords>(pageNo, pageSize);
		IPage<SeoKeywords> pageList = seoKeywordsService.page(page, queryWrapper);
		return Result.OK(pageList);
	}
	
	/**
	 *   添加
	 *
	 * @param seoKeywords
	 * @return
	 */
	@AutoLog(value = "关键词-添加")
	@Operation(summary="关键词-添加")
	@PreAuthorize("@jps.requiresPermissions('serp:seo_keywords:add')")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody SeoKeywords seoKeywords) {
		seoKeywordsService.save(seoKeywords);
		return Result.OK("添加成功！");
	}
	
	/**
	 *  编辑
	 *
	 * @param seoKeywords
	 * @return
	 */
	@AutoLog(value = "关键词-编辑")
	@Operation(summary="关键词-编辑")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords:edit')")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody SeoKeywords seoKeywords) {
		seoKeywordsService.updateById(seoKeywords);
		return Result.OK("编辑成功!");
	}
	
	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "关键词-通过id删除")
	@Operation(summary="关键词-通过id删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords:delete')")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		seoKeywordsService.removeById(id);
		return Result.OK("删除成功!");
	}
	
	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "关键词-批量删除")
	@Operation(summary="关键词-批量删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords:deleteBatch')")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.seoKeywordsService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK("批量删除成功!");
	}
	
	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "关键词-通过id查询")
	@Operation(summary="关键词-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<SeoKeywords> queryById(@RequestParam(name="id",required=true) String id) {
		SeoKeywords seoKeywords = seoKeywordsService.getById(id);
		if(seoKeywords==null) {
			return Result.error("未找到对应数据");
		}
		return Result.OK(seoKeywords);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param seoKeywords
    */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords:exportXls')")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, SeoKeywords seoKeywords) {
        return super.exportXls(request, seoKeywords, SeoKeywords.class, "关键词");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_keywords:importExcel')")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, SeoKeywords.class);
    }

	 /**
	  * 关键词排名页面 - 列表数据
	  */
	 @AutoLog(value = "营销推广-SEO-关键词排名统计查询接口")
	 @RequestMapping(value = "/keywordList")
	 public Result<?> queryRankInfoList(AvesApiSearchKeywordsDTO avesApiSearchKeywordsDTO,
										@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
										@RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
										HttpServletRequest req) throws Exception {
		 if (sysBaseAPI.isNotOwnSite(avesApiSearchKeywordsDTO.getUserFlag())) {
			 return Result.OK(new Page<>());
		 }
		 // 参数判断
		 IPage<SeoKeywords> pageList = new Page<>();
		 String userFlag = req.getParameter("userFlag");
		 if (StringUtil.isEmpty(userFlag)) {
			 userFlag = avesApiSearchKeywordsDTO.getUserFlag();
		 }
		 if (StringUtil.isEmpty(userFlag)) {
			 log.info("缺少用户flag参数");
			 return Result.OK(pageList);
		 }

		 String historyId = avesApiSearchKeywordsDTO.getHistoryId();

		 // 查询是否具有关键词优化指标
		 boolean haveAppointKeyword = false;
		 boolean haveLongTail = false;
		 QueryWrapper<SeoKpiStatistics> statisticsQueryWrapper = new QueryWrapper<>();
		 statisticsQueryWrapper.eq("host_code", userFlag);
		 statisticsQueryWrapper.eq("del_flag", 0);
		 statisticsQueryWrapper.eq("type", 1);
		 statisticsQueryWrapper.eq("history_id", historyId);
		 List<SeoKpiStatistics> seoKpiStatisticsList = seoKpiStatisticsService.list(statisticsQueryWrapper);
		 if (ListUtil.isEmpty(seoKpiStatisticsList)) {
			 return Result.OK(pageList);
		 }
		 for (SeoKpiStatistics kpiStatistics : seoKpiStatisticsList) {
			 if (kpiStatistics.getKpiTarget() == null || kpiStatistics.getKpiTarget() == 0) {
				 continue;
			 }
			 if ("APPOINT_KEYWORDS".equals(kpiStatistics.getKpiCode())) {
				 haveAppointKeyword = true;
			 } else if ("LONG_TAIL_KEYWORDS".equals(kpiStatistics.getKpiCode())) {
				 haveLongTail = true;
			 }
		 }

		 if (!haveAppointKeyword && !haveLongTail) {
			 return Result.OK(pageList);
		 }

		 Integer keywordType = avesApiSearchKeywordsDTO.getKeywordType();
		 if (haveAppointKeyword && !haveLongTail) {
			 keywordType = 1;
		 } else if (!haveAppointKeyword && haveLongTail) {
			 keywordType = 2;
		 }

		 // 查询关键词
		 String column = req.getParameter("column");
		 String order = req.getParameter("order");
		 String buttonColumn = req.getParameter("buttonColmn");
		 String buttonSort = req.getParameter("buttonSort");
		 Page<SeoKeywords> page = new Page<SeoKeywords>(pageNo, pageSize);
		 List<String> dateList = seoKeywordsService.getKeywordsLastSevenDays(userFlag,keywordType,historyId);
		 pageList = seoKeywordsService.getKeywordList(page,
				 avesApiSearchKeywordsDTO.getRankStart(),
				 avesApiSearchKeywordsDTO.getRankEnd(),
				 avesApiSearchKeywordsDTO.getKeywords(),
				 userFlag,
				 keywordType,
				 avesApiSearchKeywordsDTO.getReachStandard() != null && avesApiSearchKeywordsDTO.getReachStandard() == 1,
				 historyId,
				 column,
				 order,
				 buttonColumn,
				 buttonSort,
				 dateList);

		 // 给关键词对象增加排名信息
		 if(pageList == null){
			 return Result.error("未获取到关键词数据");
		 }
		 searchKeywordsService.addRankInfoByIntervalTimeByType(pageList.getRecords(), avesApiSearchKeywordsDTO.getKeywordType(),userFlag,historyId,dateList);
		 log.info(pageList.toString());
		 return Result.OK(pageList);
	 }

	/**
	 * Excel导出
	 * 关键词排名的导出
	 * @param avesApiSearchKeywordsDTO 查询参数
	 * @return 询盘列表
	 */
	@RequestMapping(value = "/exportExcel")
	public void exportExcel(AvesApiSearchKeywordsDTO avesApiSearchKeywordsDTO,
							HttpServletRequest req,
							HttpServletResponse response) throws Exception {
		Result res = this.queryRankInfoList(avesApiSearchKeywordsDTO, 1, 5000, req);
		if (res == null || res.getResult() == null) {
			return;
		}
		IPage<SeoKeywords> pageList = (IPage<SeoKeywords>) res.getResult();
		if (pageList.getRecords().size() == 0) {
			return;
		}
		ArrayList<String> columns = null;
		// 表头信息
		if (sysBaseAPI.isAdmin()) {
			columns = new ArrayList<>(Arrays.asList("序号", "关键词", "url", "关键词类型"));
		} else {
			columns = new ArrayList<>(Arrays.asList("序号", "关键词", "关键词类型"));
		}

		// 将关键词信息改为可以导出的格式
		List<Map<String, Object>> list = new ArrayList<>();
		List<SeoKeywords> enquiryList = pageList.getRecords();

		List<String> dates = pageList.getRecords().get(0).getDateList();
		columns.addAll(dates);

		boolean isAdmin = sysBaseAPI.isAdmin();

		for (int i = 0; i < enquiryList.size(); i++) {
			SeoKeywords seoKeywords = enquiryList.get(i);
			Map<String, Object> map = new LinkedHashMap<>();
			map.put("序号", i + 1);
			map.put("关键词", seoKeywords.getKeywords());
			if (isAdmin) {
				map.put("url", seoKeywords.getPositionUrl());
			}
			if(seoKeywords.getKeywordType() == 1) {
				map.put("关键词类型", "指定词");
			} else if(seoKeywords.getKeywordType() == 2) {
				map.put("关键词类型", "长尾词");
			}
			// 遍历日期
			for (String date : dates) {
				map.put(date,Integer.valueOf(seoKeywords.getRankInfo().get(date) == null ? "0" : seoKeywords.getRankInfo().get(date)));
			}
			list.add(map);
		}
		StringBuilder fileName = new StringBuilder();
		fileName.append("关键词排名导出");

		response.setContentType("application/vnd.ms-excel;charset=utf-8");
		// test.xls是弹出下载对话框的文件名，不能为中文，中文请自行编码
		ServletOutputStream out = null;
		ExcelWriter writer = ExcelUtil.getWriter(true);
		try {
			Sheet sheet = writer.getSheet();
			sheet.setDefaultColumnWidth(12);
			sheet.setDefaultRowHeightInPoints(20);
			// 设置请求头属性
			response.setHeader("Content-Disposition", "attachment;filename=" + new String((fileName + ".xlsx").getBytes(), StandardCharsets.ISO_8859_1));
			out = response.getOutputStream();
			writer.write(list, true);
			searchKeywordsService.setSizeColumn(sheet, list.get(0).size());
			// 写出到文件
			writer.flush(out, true);
			// 关闭writer，释放内存
			writer.close();
			// 此处记得关闭输出Servlet流
			IoUtil.close(out);
		} catch (IOException e) {
			log.error(e.getMessage());
			e.printStackTrace();
		}
	}
}
