package org.jeecg.modules.adweb.serp.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.adweb.serp.entity.ComprehensiveStatistics;
import org.jeecg.modules.adweb.serp.entity.SeoKpiStatistics;
import org.jeecg.modules.adweb.serp.service.ISearchKeywordsService;
import org.jeecg.modules.adweb.serp.service.ISeoKpiStatisticsService;
import org.jeecg.modules.adweb.site.entity.AdwebSite;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.text.ParseException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * @Description: seo_kpi_statistics @Author: jeecg-boot @Date: 2024-10-09 @Version: V1.0
 */
@Tag(name = "seo_kpi_statistics")
@RestController
@RequestMapping("/serp/seoKpiStatistics")
@Slf4j
public class SeoKpiStatisticsController
        extends JeecgController<SeoKpiStatistics, ISeoKpiStatisticsService> {
    @Autowired private ISeoKpiStatisticsService seoKpiStatisticsService;

    @Autowired private ISearchKeywordsService searchKeywordsService;
    /**
     * 分页列表查询
     *
     * @param seoKpiStatistics
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "seo_kpi_statistics-分页列表查询")
    @Operation(summary = "seo_kpi_statistics-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<SeoKpiStatistics>> queryPageList(
            SeoKpiStatistics seoKpiStatistics,
            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
            HttpServletRequest req) {
        QueryWrapper<SeoKpiStatistics> queryWrapper =
                QueryGenerator.initQueryWrapper(seoKpiStatistics, req.getParameterMap());
        Page<SeoKpiStatistics> page = new Page<SeoKpiStatistics>(pageNo, pageSize);
        IPage<SeoKpiStatistics> pageList = seoKpiStatisticsService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param seoKpiStatistics
     * @return
     */
    @AutoLog(value = "seo_kpi_statistics-添加")
    @Operation(summary = "seo_kpi_statistics-添加")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_kpi_statistics:add')")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody SeoKpiStatistics seoKpiStatistics) {
        seoKpiStatisticsService.save(seoKpiStatistics);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param seoKpiStatistics
     * @return
     */
    @AutoLog(value = "seo_kpi_statistics-编辑")
    @Operation(summary = "seo_kpi_statistics-编辑")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_kpi_statistics:edit')")
    @RequestMapping(
            value = "/edit",
            method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody SeoKpiStatistics seoKpiStatistics) {
        seoKpiStatisticsService.updateById(seoKpiStatistics);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "seo_kpi_statistics-通过id删除")
    @Operation(summary = "seo_kpi_statistics-通过id删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_kpi_statistics:delete')")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        seoKpiStatisticsService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "seo_kpi_statistics-批量删除")
    @Operation(summary = "seo_kpi_statistics-批量删除")
    @PreAuthorize("@jps.requiresPermissions('serp:seo_kpi_statistics:deleteBatch')")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.seoKpiStatisticsService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "seo_kpi_statistics-通过id查询")
    @Operation(summary = "seo_kpi_statistics-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<SeoKpiStatistics> queryById(
            @RequestParam(name = "id", required = true) String id) {
        SeoKpiStatistics seoKpiStatistics = seoKpiStatisticsService.getById(id);
        if (seoKpiStatistics == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(seoKpiStatistics);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param seoKpiStatistics
     */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_kpi_statistics:exportXls')")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, SeoKpiStatistics seoKpiStatistics) {
        return super.exportXls(
                request, seoKpiStatistics, SeoKpiStatistics.class, "seo_kpi_statistics");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @PreAuthorize("@jps.requiresPermissions('serp:seo_kpi_statistics:importExcel')")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, SeoKpiStatistics.class);
    }

    /** 获取当前用户管理的 父 站点信息 @CopyFrom: getAllSites */
    @GetMapping("/getAllSitesBySeo")
    public Result<List<AdwebSite>> getAllSitesBySeo() {
        final List<AdwebSite> allSites = seoKpiStatisticsService.getAllSites();
        return Result.OK(allSites);
    }

    /**
     * 获得综合统计信息
     */
    @GetMapping(value = "/comprehensiveInfo")
    public Result<?> getComprehensiveInfo(@RequestParam String userFlag, @RequestParam String historyId) throws ParseException {
        ComprehensiveStatistics statistics = searchKeywordsService.getComprehensiveInfo(userFlag, historyId);
        return Result.OK(statistics);
    }

    /**
     * 获取关键字排名信息
     */
    @GetMapping(value = "/getRankInfo")
    public Result<?> getRankInfo(@RequestParam("siteCode") String siteCode, @RequestParam("historyId") String historyId) throws ParseException {
        Map map = searchKeywordsService.getRankInfo(siteCode, historyId);
        return Result.OK(map);
    }
}
