package org.jeecg.modules.adweb.serp.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import jakarta.annotation.Resource;

import org.jeecg.modules.adweb.common.util.ListUtil;
import org.jeecg.modules.adweb.serp.entity.SeoKeywords;
import org.jeecg.modules.adweb.serp.mapper.SeoKeywordsMapper;
import org.jeecg.modules.adweb.serp.service.ISeoKeywordsService;
import org.jeecg.modules.adweb.serp.vo.RankInfoVO;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.List;

/**
 * @Description: 关键词
 * @Author: jeecg-boot
 * @Date:   2024-10-15
 * @Version: V1.0
 */
@Service
public class SeoKeywordsServiceImpl extends ServiceImpl<SeoKeywordsMapper, SeoKeywords> implements ISeoKeywordsService {

    @Resource private SeoKeywordsMapper seoKeywordsMapper;

    /**
     * 获取对应站点关键词最新七天时间
     * @param userFlag
     * @param keywordType
     * @param historyId
     * @return
     */
    @Override
    public List<String> getKeywordsLastSevenDays(String userFlag, Integer keywordType, String historyId){
        //获取对应站点最近七天
        List<String> dateList = seoKeywordsMapper.getKeywordsLastSevenDays(userFlag, historyId,keywordType);
        return dateList;
    }

    /**
     * 获取一个站点的达标关键词排名数量
     *
     * @param siteCode 站点code
     * @return 关键词排名数量
     */
    @Override
    public RankInfoVO getSeoRankInfo(String siteCode, String historyId, Integer rankStart, Integer rankEnd, String dateStr, String keywordType) {
        //指定词
        return seoKeywordsMapper.queryKeywordNumByRankRange(siteCode, historyId,rankStart,rankEnd,dateStr,keywordType);
    }

    /**
     * 获取关键词列表
     * @return 关键词列表
     */
    @Override
    public IPage<SeoKeywords> getKeywordList(IPage<SeoKeywords> page,
                                             Integer rankStart,
                                             Integer rankEnd,
                                             String keyword,
                                             String siteCode,
                                             Integer keywordType,
                                             boolean reachStandard,
                                             String historyId,
                                             String column,
                                             String order,
                                             String buttonColumn,
                                             String buttonSort,
                                             List<String> dateList) throws ParseException {
        if (reachStandard) {
            rankStart = 1;
            int standard = 10;
            if (keywordType != null && keywordType == 1) {
                standard = seoKeywordsMapper.getKeywordStandard(siteCode);
            }
            rankEnd = standard;
        }

        boolean isCustomer = true;

        String datesStr = "";
        if(ListUtil.isEmpty(dateList)){
            return null;
        }else{
            for(int i = 0; i < dateList.size(); i++){
                if(i == dateList.size() - 1){
                    datesStr +=  "'" + dateList.get(i) + "'";
                }else{
                    datesStr +=  "'" + dateList.get(i) + "'" + ",";
                }
            }
            datesStr = "(" + datesStr + ")";
        }

        return seoKeywordsMapper.getKeywordList(page, rankStart, rankEnd, isCustomer, keyword, siteCode, keywordType, datesStr, historyId,column,order,buttonColumn,buttonSort);
    }
}
